const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

var firefox = Application.id == "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
var thunderbird = Application.id == "{3550f703-e582-4d05-9a08-453d09bdfdc6}";

if (!thunderbird) {
	var contentPrefs = Cc ["@mozilla.org/content-pref/service;1"].getService (Ci.nsIContentPrefService);
	var db = contentPrefs.DBConnection;
	var ioService = Cc ["@mozilla.org/network/io-service;1"].getService (Ci.nsIIOService);
	var siteList = document.getElementById ('sitelist');
}

var sitesPane = document.getElementById ('shrunked-sites');
var defaultsPane = document.getElementById ('shrunked-defaults');

var maxWidthPref = document.getElementById ('maxwidth');
var maxHeightPref = document.getElementById ('maxheight');
var qualityPref = document.getElementById ('quality');

var noresize = document.getElementById ('noresize');
var small = document.getElementById ('small');
var medium = document.getElementById ('medium');
var large = document.getElementById ('large');
var custom = document.getElementById ('custom');
var label1 = document.getElementById ('label1');
var customvalue1 = document.getElementById ('customvalue1');
var label2 = document.getElementById ('label2');
var customvalue2 = document.getElementById ('customvalue2');
var slider = document.getElementById ('slider');

var strings = document.getElementById ('shrunked-strings');

(function () {
	if (firefox) {
		noresize.collapsed = true;
	}
	if (thunderbird) {
		try {
			document.documentElement.showPane (defaultsPane);
		} catch (e) {
			Cu.reportError (e);
		}
		sitesPane.setAttribute ('collapsed', 'true');
		var selector = document.getAnonymousElementByAttribute (document.documentElement, 'anonid', 'selector');
		selector.setAttribute ('collapsed', 'true');
	} else {
		var statement = db.createStatement ("SELECT prefs.value, settings.name, groups.name AS site " +
			"FROM prefs, settings, groups WHERE prefs.groupID = groups.id " +
			"AND prefs.settingID = settings.id AND settings.name LIKE 'extensions.shrunked.%' ORDER BY groups.name ASC");

		var data = [];
		while (statement.executeStep ()) {
			var value;
			switch (statement.getTypeOfIndex (0)) {
        		case 1:
					value = statement.getInt32 (0);
					break;
				case 3:
					value = statement.getString (0);
					break;
			}
			var name = statement.getString (1);
			var site = statement.getString (2);

			if (!data [site]) {
				data [site] = new Array ();
			}
			data [site][name] = value;
		}
		handleData (data);
	}

	var maxWidth = maxWidthPref.value;
	var maxHeight = maxHeightPref.value;

	if (maxWidth == -1 && maxHeight == -1) {
		medium.parentNode.selectedIndex = 0;
	} else if (maxWidth == 500 && maxHeight == 500) {
		medium.parentNode.selectedIndex = 1;
	} else if (maxWidth == 800 && maxHeight == 800) {
		medium.parentNode.selectedIndex = 2;
	} else if (maxWidth == 1200 && maxHeight == 1200) {
		medium.parentNode.selectedIndex = 3;
	} else {
		medium.parentNode.selectedIndex = 4;
		customvalue1.value = maxWidthPref.value;
		customvalue2.value = maxHeightPref.value;
	}

	slider.value = qualityPref.value;

	validate ();
})();

function handleData (data) {
	for (var site in data) {
		var disabled =  data [site]['extensions.shrunked.disabled'];
		var maxWidth = data [site]['extensions.shrunked.maxWidth'];
		var maxHeight = data [site]['extensions.shrunked.maxHeight'];
		if (disabled) {
			var item = document.createElement ('listitem');
			item.setAttribute ('style', 'color: #666; font-style: italic');
			var siteCell = document.createElement ('listcell');
			siteCell.setAttribute ('label', site);
			item.appendChild (siteCell);
			var disabledCell = document.createElement ('listcell');
			disabledCell.setAttribute ('label', strings.getString ('disabled'));
			disabledCell.setAttribute ('style', 'text-align: center');
			item.appendChild (disabledCell);
			siteList.appendChild (item);
		}
		if (maxWidth && maxHeight) {
			var item = document.createElement ('listitem');
			var siteCell = document.createElement ('listcell');
			siteCell.setAttribute ('label', site);
			item.appendChild (siteCell);
			var widthCell = document.createElement ('listcell');
			widthCell.setAttribute ('label', strings.getFormattedString ('dimensions', [maxWidth, maxHeight]));
			widthCell.setAttribute ('style', 'text-align: center');
			item.appendChild (widthCell);
			siteList.appendChild (item);
		}
	}
}

function doForget () {
	var item = siteList.getSelectedItem (0);
	if (item) {
		var site = item.firstChild.getAttribute ('label');
		var u = ioService.newURI ('http://' + site + '/', null, null);

		contentPrefs.removePref (u, 'extensions.shrunked.maxHeight');
		contentPrefs.removePref (u, 'extensions.shrunked.maxWidth');
		contentPrefs.removePref (u, 'extensions.shrunked.disabled');
	}
	siteList.removeChild (item);
}

function validate () {
	label1.disabled = customvalue1.disabled = label2.disabled = customvalue2.disabled = !custom.selected;
	if (noresize.selected) {
		maxWidthPref.value = -1;
		maxHeightPref.value = -1;
	} else if (small.selected) {
		maxWidthPref.value = 500;
		maxHeightPref.value = 500;
	} else if (medium.selected) {
		maxWidthPref.value = 800;
		maxHeightPref.value = 800;
	} else if (large.selected) {
		maxWidthPref.value = 1200;
		maxHeightPref.value = 1200;
	} else {
		maxWidthPref.value = customvalue1.value;
		maxHeightPref.value = customvalue2.value;
	}
}
